<?php

namespace UBRIR;

class NewCertificates implements UsingNavigator
{
    public function __construct(InterfaceConnectionConfig $connector = null)
    {
        if ($connector !== null) {
            Core::GetCore()->PlugInCMSConnector($connector);
        }
    }

    //region UsingNavigator
    private $cmsNavigator;

    public function GetNavigator()
    {
        return $this->cmsNavigator;
    }

    public function UseNavigator(CMSNavigator $cmsNavigator)
    {
        $this->cmsNavigator = $cmsNavigator;
    }
    //endregion


    /**
     * @param $merchantId
     * @param $managerEmail
     * @param $passPhrase
     * @param string $country
     * @param string $region
     * @param string $city
     * @param string $organization
     * @param string $division
     * @return DataContainer
     */
    function Generate($merchantId, $managerEmail, $passPhrase = '', $country = 'RU', $region = 'SVO', $city = 'MSK', $organization = 'Some Organization', $division = 'Main')
    {
        $responseData = new DataContainer();
        $connector = Core::GetCore()->GetCMSConnector();

        $arReceivers = $connector->GetBankReceiversEmailList();

        if (!is_array($arReceivers)) {
            $responseData->AddError('Неверный формат получателей почты. Можно использовать массив получателей или строку с разделитилем [,]');
            return $responseData;
        }

        if (empty($merchantId)) {
            $responseData->AddError('Не указан идентификатор магазина (MerchantID)');
            return $responseData;
        }

        if (empty($managerEmail)) {
            $responseData->AddError('Неверный email ответственного лица');
            return $responseData;
        }

        if (empty($passPhrase)) {
            $responseData->AddError('Не указан пароль от сертификата. Укажите пароль');
            return $responseData;
        }


        //Запись $merchantId если текущий пустой
        $currentShopId = $connector->GetShopId();
        if (empty($currentShopId)) {
            $connector->SetShopId($merchantId);
        }

        //При генерации сохраняем новый пароль отдельного от работающего. После заливки .pem мы будем использовать его
        $connector->SetNewPrivateCertificatePassword($passPhrase);


        ob_start(); //Защищаем вывод от Warning'ов в процессе генерации
        //Генерируем приватный ключ
        $privateKeyPath = $this->getNewPrivateCertificateFilePath();
        $command = 'openssl genrsa -des3 -passout pass:' . escapeshellarg($passPhrase) . ' -out ' . escapeshellarg($privateKeyPath) . ' 2048';
        $result = false;
        $output = '';
        exec($command, $output, $result);

        //$keys = openssl_pkey_new();

        //Проверка ключа
        if ($result) {
            $responseData->AddError('Возникла проблема с генерацией ключа');
            return $responseData;
        }

        //Читаем приватный ключ
        $privateKeyText = openssl_pkey_get_private('file://' . $privateKeyPath, $passPhrase);
        if ($privateKeyText === false || filesize($privateKeyPath) == 0) {
            $responseData->AddError('Не удалось сгенерировать новый приватный ключ');
            return $responseData;
        }

        //Вносим переменные в массив для генерации
        $requestData = array(
            "countryName" => $country,
            "stateOrProvinceName" => $region,
            "localityName" => $city,
            "organizationName" => $organization,
            "organizationalUnitName" => $division,
            "commonName" => $merchantId,
            "emailAddress" => $managerEmail
        );

        //Генерируем запрос
        $request = openssl_csr_new($requestData, $privateKeyText, Array("digest_alg" => "sha256"));

        //Проверка запроса
        if (!$request) {
            $responseData->AddError('Возникла проблема с генерацией запроса');
            return $responseData;
        }
        $connector->UpdateCertificateCreateDate();

        //Помещаем содержимое запроса в файл
        $requestPath = $this->getRequestFilePath();
        openssl_csr_export($request, $requestText);
        $requestFile = fopen($requestPath, 'w');
        fwrite($requestFile, $requestText);
        fclose($requestFile);

        //Данные для отправки почты
        $from = $managerEmail;
        $subject = 'Запрос на создание сертификата';
        $subject = MailNotifications::UTF8_Encode($subject);
        $message = 'Файл запроса во вложении';

        //Имя вложенного файла
        $filename = 'request.csr';

        //Разделитель
        $boundary = "---";

        //Заголовки
        $headers = "From: $from\nReply-To: $from\n";
        $headers .= "Content-Type: multipart/mixed; boundary=\"$boundary\"";
        $body = "--$boundary\n";

        /* Присоединяем текстовое сообщение */
        $body .= "Content-type: text/html; charset=\"utf-8\"\n";
        $body .= "Content-Transfer-Encoding: quoted-printablenn";
        $body .= "Content-Disposition: attachment; filename==?utf-8?B?" . base64_encode($filename) . "?=\n\n";
        $body .= $message . "\n";
        $body .= "--$boundary\n";

        //Считываем файл csr запроса
        $file = fopen($requestPath, "r");
        $text = fread($file, filesize($requestPath));
        fclose($file);

        // Добавляем тип содержимого, кодируем текст файла и добавляем в тело письма
        $body .= "Content-Type: application/octet-stream; name==?utf-8?B?" . base64_encode($filename) . "?=\n";
        $body .= "Content-Transfer-Encoding: base64\n";
        $body .= "Content-Disposition: attachment; filename==?utf-8?B?" . base64_encode($filename) . "?=\n\n";
        $body .= chunk_split(base64_encode($text)) . "\n";
        $body .= "--" . $boundary . "--\n";
        //Отправляем письма
        $mail = true;
        foreach ($arReceivers as $to) {
            $mail = mail($to, $subject, $body, $headers);
            if (!$mail) {
                $responseData->AddError('Возникла проблема с отправкой письма на адрес ' . $to);
            }
        }

        if ($request && $mail) {
            $responseData->setSuccess();
            $responseData->AddMessage('Запрос успешно отправлен в банк', 'success');
        }
        ob_get_clean();

        return $responseData;
    }

    public function getRequestFilePath()
    {
        return Helpers::GetUbrrCoreDir() . '/certs/request.csr';
    }

    public function getNewPrivateCertificateFilePath()
    {
        $privateKeyPath = Helpers::GetUbrrCoreDir() . '/certs/_new.key';
        return $privateKeyPath;
    }

    public function getPrivateCertificateFilePath()
    {
        $validNames = Core::GetCore()->Certificates_DefaultPrivateFiles();

        return $validNames[0];
    }

    public function getPublicCertificateFilePath()
    {
        $validNames = Core::GetCore()->Certificates_DefaultPublicFiles();

        return $validNames[0];
    }

    public function getNewPublicCertificateFilePath()
    {
        $shopId = Core::GetCore()->GetShopId();

        $certificatesDir = Helpers::GetUbrrCoreDir() . '/certs/';

        $newPublicCertificateFilePath = $certificatesDir . '_new_' . $shopId . '.pem';

        return $newPublicCertificateFilePath;
    }

    public function CheckCertificatesModulus()
    {
        $core = Core::GetCore();

        $core->Log('Проверка сертификатов:');

        $responseData = new DataContainer(true);

        $publicCertificate = $this->getNewPublicCertificateFilePath();
        $privateCertificate = $this->getNewPrivateCertificateFilePath();
        $privatePass = $core->GetCMSConnector()->GetNewPrivateCertificatePassword();

        $getPublicModulusCommand = 'openssl x509 -noout -modulus -in ' . escapeshellarg($publicCertificate);
        $result = false;
        $publicModulus = '';
        exec($getPublicModulusCommand, $publicModulus, $result);
        $publicModulus = $publicModulus[0];
        $core->Log('Публичный:' . "\t" . $publicModulus);

        if (!file_exists($privateCertificate)) {
            $responseData->AddError('Файл приватного ключа ' . $privateCertificate . ' не найден!');
        } else {
            $getPrivateModulusCommand = 'openssl rsa -noout -modulus -in ' . escapeshellarg($privateCertificate) . ' -passin pass:' . escapeshellarg($privatePass);
            $result = false;
            $privateModulus = '';
            exec($getPrivateModulusCommand, $privateModulus, $result);
            if (is_array($privateModulus) && count($privateModulus) > 0) {
                $privateModulus = $privateModulus[0];
                $core->Log('Приватный:' . "\t" . $privateModulus);
            } else {
                $core->Log('Не удалось получить модуль приватного ключа');
                $privateModulus = false;
            }

            if ($publicModulus != $privateModulus) {
                $responseData->AddError('Публичный сертификат не соответсвтует приватному. Использование не возможно. Проверьте правильный и публичный ключ Вы использовали.');
            }
        }

        return $responseData;
    }

    public function TestAndReplaceWithNewCertificates()
    {
        $core = Core::GetCore();

        $publicCertificate = $this->getNewPublicCertificateFilePath();
        $privateCertificate = $this->getNewPrivateCertificateFilePath();
        $privatePass = $core->GetCMSConnector()->GetNewPrivateCertificatePassword();
        $core->ServerConnectionConfig(true, $publicCertificate, $privateCertificate, $privatePass);

        $responseData = $this->CheckCertificatesModulus();

        if ($responseData->isSuccess()) {
            $gatewayAdminPanel = new GatewayForAdmin();
            $responseData = $gatewayAdminPanel->GetJournal();

            if ($responseData->isSuccess()) {
                //Заменяем старые сертификаты
                if (!rename($privateCertificate, $this->getPrivateCertificateFilePath())) {
                    $responseData->AddError('Не удалось скопировать новый приватный ключ');
                }

                if (!rename($publicCertificate, $this->getPublicCertificateFilePath())) {
                    $responseData->AddError('Не удалось скопировать новый публичный ключ');
                }

                //Удаляем файл запроса
                if (file_exists($this->getRequestFilePath()))
                    unlink($this->getRequestFilePath());

                //При копировании не было ошибок
                if ($responseData->isSuccess()) {
                    //Устанавливаем новый пароль приватного ключа
                    $core->GetCMSConnector()->SetPrivateCertificatePassword($privatePass);
                }
            } else {
                $responseData->AddError('Не удалось исползовать сертификаты для взаимодействия с платежным шлюзом. Свяжитесь с банком или попробуйте перевыпустить сертификат.');
            }
        }

        return $responseData;
    }

    public function GetCertificateUploadPage()
    {
        $ubrirCertificateGenerator = $this;
        $testStatusData = new DataContainer();

        if ($_SERVER['REQUEST_METHOD'] === 'POST') {
            $destination = $this->getNewPublicCertificateFilePath();

            if (move_uploaded_file($_FILES['new_public_certificate']['tmp_name'], $destination)) {
                $testStatusData = $this->TestAndReplaceWithNewCertificates();
            } else {
                $testStatusData->AddError('Не удалось переместить загруженный сертификат. Обратитесь к администратору');
            }
        }

        $view_data = Array(
            'ubrirCertificateGenerator' => $ubrirCertificateGenerator,
            'testStatusData' => $testStatusData,
        );

        $result = Core::GetCore()->IncludeTemplate('view.certificate_upload', $view_data);

        return $result;
    }

    public function GetCertificateGeneratePage()
    {
        $core = Core::GetCore();

        $ubrirCertificateGenerator = $this;

        $connector = $core->GetCMSConnector();

        $arCountryList = TypeEnums::GetCountryList();
        $arRuRegionsList = TypeEnums::GetRuRegionList();
        $arRuCityList = TypeEnums::GetRuCityList();

        $shopId = $connector->GetShopId();
        $newCertificatePassword = $connector->GetNewPrivateCertificatePassword();

        $resultText = '';
        $responseData = new DataContainer();

        if ($_SERVER['REQUEST_METHOD'] === 'POST') {
            $showForm = false;

            $organization = $_POST['organization'];
            if (empty($organization)) {
                $resultText = 'Незаполненно поле "Организация"' . "\r\n";
                $showForm = true;
            }

            $merchantId = intval(trim($_POST['merchant_id']));
            if (!($merchantId > 0)) {
                $valueErrors['merchant_id'] = 'Неверный формат номера ретейлера! (Он должен состоять исключительно из чисел!)';
                $resultText = '<div class="error">Неверный формат номера ретейлера! (Он должен состоять исключительно из чисел!)</div>';
                $showForm = true;
            }

            $from_email = $_POST['from_email'];
            if (empty($from_email) || !Helpers::CheckEmail($from_email)) {
                $valueErrors['from_email'] = 'Неверный формат email ответственного лица!';
                $resultText = '<div class="error">Неверный формат email ответственного лица!</div>';
                $showForm = true;
            }

            $password = $_POST['password'];

            $country = $_POST['country'];
            $region = $_POST['region'];
            $city = $_POST['city'];

            $division = $_POST['division'];

            //Необходимости рисовать форму не возникло - значит генерируем сертификаты
            if (!$showForm) {
                if (!empty($organization)) {
                    $organization = Helpers::Transliterate($organization);
                }
                
                if (!empty($division)) {
                    $division = Helpers::Transliterate($division);
                }
                else{
                    $division = 'Sales';
                }

                if (!empty($country)) {
                    if (isset($arCountryList[$country]))
                        $country = $arCountryList[$country];
                    else
                        $country = Helpers::Transliterate($country);
                }

                if (!empty($region)) {
                    if (isset($arRuRegionsList[$region]))
                        $region = $arRuRegionsList[$region];
                    else
                        $region = Helpers::Transliterate($region);
                }

                if (!empty($city)) {
                    $city = Helpers::Transliterate($city);
                }

                $responseData = $this->Generate($merchantId, $from_email, $password, $country, $region, $city, $organization, $division);

                $arMessages = $responseData->GetMessages();

                if ($responseData->isSuccess()) {
                    $resultText = '<div class="success">' . $arMessages['success'][0] . '</div>';
                } else {
                    $resultText = '<div class="error">' . $arMessages['ERROR'][0] . '</div>';
                }
            }
        } else {
            $showForm = true;
        }


        $view_data = Array(
            'ubrirCertificateGenerator' => $ubrirCertificateGenerator,
            'resultText' => $resultText,
            'showForm' => $showForm,
            'shopId' => $shopId,
            'newCertificatePassword' => $newCertificatePassword,
            'arCountryList' => $arCountryList,
            'arRuRegionsList' => $arRuRegionsList,
            'arRuCityList' => $arRuCityList,
            'generationStatusData' => $responseData
        );

        $result = $core->IncludeTemplate('view.certificate_generate', $view_data);

        return $result;


    }
}